@@ -8,8 +8,8 @@ from ..calibration_data import (
 
 def test_load_MAST_remote():
     sp = load_MAST_calspec("g191b2b_005.fits", remote=True, show_progress=False)
-    assert(sp is not None)
-    assert(len(sp.spectral_axis) > 0)
+    assert sp is not None
+    assert len(sp.spectral_axis) > 0
 
 
 def test_load_MAST_local():
@@ -19,11 +19,11 @@ def test_load_MAST_local():
         pkgname='specreduce'
     )
     sp = load_MAST_calspec(sp_file, remote=False)
-    assert(sp is not None)
-    assert(len(sp.spectral_axis) > 0)
+    assert sp is not None
+    assert len(sp.spectral_axis) > 0
 
 
 def test_load_onedstds():
     sp = load_onedstds()
-    assert(sp is not None)
-    assert(len(sp.spectral_axis) > 0)
+    assert sp is not None
+    assert len(sp.spectral_axis) > 0
